//
//  CustomOutlineViewAppDelegate.m
//  CustomOutlineView
//
//  Created by Steven Streeting on 07/08/2010.
//  Copyright 2010 Torus Knot Software Ltd. 
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following condition:
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "CustomOutlineViewAppDelegate.h"
#import "MyTreeNode.h"
#import "CustomCellController.h"
#import "CustomCell.h"

@implementation CustomOutlineViewAppDelegate

@synthesize window;
@synthesize outlineView;

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification 
{
	// Insert code here to initialize your application 
	
	cellViewControllers = [[NSMutableDictionary alloc] init];
	
	CustomCell* cell = [[CustomCell alloc] init];
	[cell setEditable:YES];
	
	[[[outlineView tableColumns] objectAtIndex:0] setDataCell:cell];
	
	treeModel = [[NSMutableArray alloc] initWithObjects:
				 [[MyTreeNode alloc] initGroupWithName:@"My First Group"], 
				 [[MyTreeNode alloc] initGroupWithName:@"My Second Group"],
				 [[MyTreeNode alloc] initGroupWithName:@"My Third Group"],
				 nil
				 ];
	
	for (MyTreeNode* parent in treeModel)
	{
		for (unsigned int i = 0; i < 3; ++i)
		{
			NSString* name = [NSString stringWithFormat:@"Detail item %u", i+1];
			MyTreeNode* child = [[MyTreeNode alloc] initWithName:name];
			switch(i)
			{
				case 0:
					child.someText = @"Lorem ipsum dolor sit amet, consectetur adipisicing elit, "
					  "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim "
					  "ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat.";
					child.optionValue = 0;
					child.boolValue = YES;
					break;
				case 1:
					child.someText = @"Mary had a little lamb, whose fleece was white as snow. \n"
					  "And everywhere that Mary went, the lamb was sure to go.";					
					child.optionValue = 1;
					child.boolValue = NO;
					break;
				case 2:
					child.someText = @"General Kenobi: Years ago, you served my father in the Clone Wars; "
					  "now he begs you to help him in his struggle against the Empire. I regret that I am "
					  "unable to present my father's request to you in person; but my ship has fallen under "
					  "attack and I'm afraid my mission to bring you to Alderaan has failed. I've placed "
					  "information vital to the survival of the rebellion into the memory systems of this R2 unit. "
					  "My father will know how to retrieve it. You must see this droid safely delivered to him on Alderaan. "
					  "This is our most desperate hour. Help me, Obi-Wan Kenobi; you're my only hope.";
					child.optionValue = 2;
					child.boolValue = NO;
					break;
					
			}
			[parent insertObject:child inChildrenAtIndex:i];
		}
	}
	
	
	[outlineView reloadData];
	
	
	
}

#pragma mark -
#pragma mark OutlineView datasource/delegate
-(NSInteger)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
	if (!item)
		return [treeModel count];
	else
	{
		MyTreeNode* node = item;
		return [[node children] count];
	}
	
}
-(BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
	MyTreeNode* node = item;
	return [node isGroup];
}

- (void)outlineView:(NSOutlineView *)oview willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
	// It's at this stage that we can allocate the specific view for the cell
	MyTreeNode* node = item;
	
	CustomCellController* cellCtrl = [cellViewControllers objectForKey:node];
	if (!cellCtrl)
	{
		cellCtrl = [[CustomCellController alloc] initWithNode:node];
		[cellViewControllers setObject:cellCtrl forKey:node];
	}
	
	CustomCell* mycell = (CustomCell*)cell;
	
	[mycell setController: cellCtrl];
	
	
}

-(void)hideViewsForItem:(MyTreeNode*)node
{
	CustomCellController* cellCtrl = [cellViewControllers objectForKey:node];
	if (cellCtrl)
	{
		[cellCtrl hideViews];
	}
	
	for (MyTreeNode* child in node.children)
		[self hideViewsForItem:child];
	
	
}
-(void)removeItems:(MyTreeNode*)node
{
	for (MyTreeNode* child in node.children)
		[self removeItems:child];
	
	CustomCellController* cellCtrl = [cellViewControllers objectForKey:node];
	if (cellCtrl)
	{
		[cellCtrl hideViews];
		[cellViewControllers removeObjectForKey:node];
	}
	
	
}

-(void)outlineViewItemDidCollapse:(NSNotification*)notif
{
	// we need to implement this to hide the collapsed views
	MyTreeNode* node = [[notif userInfo] objectForKey:@"NSObject"];
	
	// The collapsed item is the parent of the nodes that disappear
	for (MyTreeNode* child in node.children)
		[self hideViewsForItem:child];
		
}


-(id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
	MyTreeNode* node = item;
	return node;
}

-(id)outlineView:(NSOutlineView *)outlineView child:(NSInteger)index ofItem:(id)item
{
	if (item)
	{
		MyTreeNode* node = item;
		return [[node children] objectAtIndex:index];
	}
	else
		return [treeModel objectAtIndex:index];
}


-(CGFloat)outlineView:(NSOutlineView *)outlineView heightOfRowByItem:(id)item
{
	MyTreeNode* node = item;
	
	if ([node isGroup])
		return 17;
	else
		return 65;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldEditTableColumn:(NSTableColumn*)col item:(id)item
{
	return YES;
}

-(void)outlineView:(NSOutlineView *)outlineView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
	// required to set the value when editing
	MyTreeNode* node = item;
	
	node.name = object;
	
}

- (BOOL)control:(NSControl *)control textShouldEndEditing:(NSText *)fieldEditor
{
	if ([[fieldEditor string] length] == 0)
	{
		// don't allow empty node names
		return NO;
	}
	else
	{
		return YES;
	}
}


@end
