//
//  ModelTreeNode.m
//  CustomOutlineView
//
//  Created by Steven Streeting on 07/08/2010.
//  Copyright 2010 Torus Knot Software Ltd. 
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following condition:
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "MyTreeNode.h"


@implementation MyTreeNode

@synthesize someText;
@synthesize boolValue;
@synthesize optionValue;
@synthesize hashValue;

- (id)init
{
	if (self = [super init])
	{
		self.someText = @"";
		self.boolValue = NO;
		self.optionValue = 0;
		
		// Generate a hash for this instance and copies of it
		// This is a bit hacky and you might not want to do this, it makes the
		// instance unique and easy to store as a hash value for comparison / indexing
		hashValue = [[[NSProcessInfo processInfo] globallyUniqueString] hash];

	}
	return self;
}

#pragma mark -
#pragma mark NSCopying protocol
- (id)copyWithZone:(NSZone*)zone
{
	MyTreeNode *copy = [[MyTreeNode allocWithZone:zone] init];
	if (copy)
	{
		copy.name = self.name;
		copy.isGroup = self.isGroup;
		if (self.isGroup)
			copy.children = self.children;
		
		copy.someText = self.someText;
		copy.boolValue = self.boolValue;
		copy.optionValue = self.optionValue;
		copy.hashValue = self.hashValue;
		
	}
	return copy;
}

#pragma mark -
#pragma mark NSCoding protocol

- (NSArray*)keysForEncoding
{
	NSArray* baseKeys = [super keysForEncoding];
	NSArray* myKeys = [NSArray arrayWithObjects: @"someText", @"boolValue", @"optionValue", nil];
	return [baseKeys arrayByAddingObjectsFromArray:myKeys];
}
- (void)setNilValueForKey:(NSString *)key
{
	// overridden to deal with primitives
	if ([key isEqualToString:@"boolValue"])
		self.boolValue = NO;
	else if ([key isEqualToString:@"optionValue"])
		self.optionValue = 0;
	else 
		[super setNilValueForKey:key];
	
}

-(NSUInteger) hash
{
	return hashValue;
	
	
}

-(BOOL) isEqual:(id)object
{
	
	// isEqual must agree with hash, and neither can be based on mutable state
	if ([object class] == [self class])
	{
		MyTreeNode* oth = object;
		return hashValue == oth.hashValue;
	}
	
	return NO;
	
}

@end
